@@ -203,11 +203,11 @@ See also the `profile' slot in the `browser' class.")
             `(pre
               :font-family ,theme:monospace-font-family
               :font-size "0.9rem"
-              :border-radius "3px"
+              :border-radius "2px"
               :overflow "auto"
               :padding "5px")
             `("table"
-              :border-radius "3px"
+              :border-radius "2px"
               :border-spacing "0"
               :width "100%")
             `("pre, p code"
@@ -231,13 +231,13 @@ See also the `profile' slot in the `browser' class.")
               :color ,theme:on-primary
               :text-align "left")
             `("th:first-of-type"
-              :border-top-left-radius "3px")
+              :border-top-left-radius "1px")
             `("th:last-of-type"
-              :border-top-right-radius "3px")
+              :border-top-right-radius "1px")
             `("tr:last-of-type td:first-of-type"
-              :border-bottom-left-radius "3px")
+              :border-bottom-left-radius "2px")
             `("tr:last-of-type td:last-of-type"
-              :border-bottom-right-radius "3px")
+              :border-bottom-right-radius "2px")
             `("table.resizable-table th"
               :resize "horizontal"
               :overflow "auto")
@@ -275,7 +275,7 @@ See also the `profile' slot in the `browser' class.")
               :padding "4px"
               :padding-left "8px"
               :padding-right "8px"
-              :border-radius "3px"
+              :border-radius "2px"
               :font-weight "bold"
               :background-color ,theme:background-alt)
             `("dd"
@@ -645,15 +645,12 @@ which is not necessarily the same."))
   (:export-accessor-names-p t)
   (:export-predicate-name-p t)
   (:metaclass user-class)
-  (:documentation "A buffer which focus sets the current context, that is, the
-buffer-specific values of various settings like the various file paths, prompt
-options, download options, etc.
+  (:documentation "A buffer that holds buffer-specific settings (see its slots).
 
-Every setting that can be buffer-specific should be stored here; settings that
-only make sense globally should be stored in `browser' instead.
+Global settings should be stored in `browser' instead.
 
-It's similar to the \"private window\" in popular browser, but the scope here is
-the buffer (which gives us more flexibility)."))
+Conceptually, it's similar to \"private windows\" in popular browsers but the
+scope is that of buffers."))
 
 (defmethod print-object ((buffer buffer) stream)
   (print-unreadable-object (buffer stream :type t :identity t)
@@ -691,10 +688,7 @@ Nsymbols will find the proper symbol, unless duplicate."
 (define-class network-buffer (buffer)
   ((status
     :unloaded
-    :type (member :loading
-                  :finished
-                  :unloaded
-                  :failed)
+    :type (member :loading :finished :unloaded :failed)
     :export nil ; TODO: Need to decide if we want progress / errors before exposing to the user.
     :documentation "The status of the buffer.
 - `:loading' when loading a web resource.
@@ -743,8 +737,6 @@ The current buffer URL should not be relied upon.  With WebKitGTK, it is the sam
 as (url REQUEST-DATA).
 If you need to access the URL before this request, inspect the document-mode history.
 
-There's no more ability to pass the results to the renderer with :FORWARD.
-
 Example:
 
 \(defmethod configure-instance ((buffer buffer))
@@ -927,7 +919,7 @@ store them somewhere and `ffi-buffer-delete' them once done."))
             `("a:visited"
               :color ,theme:secondary)
             `(".progress-bar-container"
-              :border-radius "3px"
+              :border-radius "2px"
               :height "20px"
               :width "100%")
             `(".progress-bar-base"
@@ -1189,11 +1181,10 @@ identifiers."
 (defun dead-buffer-p (buffer)           ; TODO: Use this wherever needed.
   (not (buffers-get (id buffer))))
 
-(-> resurrect-buffer (buffer) (values &optional buffer))
+(-> resurrect-buffer (buffer))
 (defun resurrect-buffer (dead-buffer)
   ;; (setf (id dead-buffer) (new-id))      ; TODO: Shall we reset the ID?
-  (ffi-buffer-make dead-buffer)
-  dead-buffer)
+  (ffi-buffer-make dead-buffer))
 
 (define-generic document-model ((buffer buffer))
   "A wraparound accessor to BUFFER's `document-model'.
@@ -1293,7 +1284,8 @@ To load nothing, set it to 'about:blank'.
 PARENT-BUFFER is useful when we want to record buffer- and history relationships.
 LOAD-URL-P controls whether to load URL right at buffer creation."
   (let* ((url (url url))
-         (buffer (apply #'make-instance buffer-class
+         (buffer (apply #'make-instance
+                        buffer-class
                         :title title
                         :extra-modes modes
                         :parent-buffer parent-buffer
@@ -1358,9 +1350,10 @@ See `make-buffer' for a description of the arguments."
 
 (-> add-to-recent-buffers (buffer) *)
 (defun add-to-recent-buffers (buffer)
-  "Create a recent-buffer from given buffer and add it to `recent-buffers'."
+  "Push BUFFER to the front of `recent-buffers'.
+The notion of first element is dictated by `containers:first-item'."
   (when (web-buffer-p buffer)
-    (containers:delete-item-if (recent-buffers *browser*) (buffer-match-predicate buffer))
+    (containers:delete-item (recent-buffers *browser*) buffer)
     (containers:insert-item (recent-buffers *browser*) buffer)))
 
 (export-always 'buffer-delete)
@@ -1398,30 +1391,31 @@ This is a low-level function.  See `buffer-delete' for the high-level version."
 (export-always 'buffer-list)
 (defun buffer-list ()
   "Order is stable."
-  (sort
-   (alex:hash-table-values (buffers *browser*))
-   #'>
-   ;; TODO: Sort by creation time instead?
-   :key #'id))
+  (sort (alex:hash-table-values (buffers *browser*))
+        #'>
+        :key #'id))
 
 (defun buffers-get (id)
   "Get the `buffer' with the corresponding ID."
   (gethash id (slot-value *browser* 'buffers)))
 
 (defun buffers-set (id buffer)
-  "Set the BUFFER as the one corresponding to ID in `browser'."
+  "Ensure that entry ID->BUFFER belongs to `buffers' hash table."
   (when *browser*
+    ;; Mutate state of the hash table.
     (setf (gethash id (slot-value *browser* 'buffers)) buffer)
-    ;; Force setf call so that slot is seen as changed, e.g. by status buffer watcher.
+    ;; Notify `buffers' of the new hash table state. Useful, for example, to
+    ;; update the status buffer.
     (setf (buffers *browser*) (buffers *browser*))))
 
 (defun buffers-delete (id)
-  "Remove the buffer with respective ID from the browser.
+  "Remove `buffers' hash table entry matching key ID.
 
-Low-level function, use `buffer-delete' to properly close the buffer, or
-`delete-buffer' command from inside Nyxt."
+This is a low-level function.  See `buffer-delete' and `delete-buffer'."
+  ;; Mutate state of the hash table.
   (remhash id (slot-value *browser* 'buffers))
-  ;; Force setf call so that slot is seen as changed, e.g. by status buffer watcher.
+  ;; Notify `buffers' of the new hash table state. Useful, for example, to
+  ;; update the status buffer.
   (setf (buffers *browser*) (buffers *browser*)))
 
 (export-always 'window-list)
@@ -1479,20 +1473,17 @@ proceeding."
     (setf (last-access buffer) (time:now)))
   (when (and (network-buffer-p buffer)
              (eq (slot-value buffer 'status) :unloaded))
-    (reload-buffer buffer)))
+    (buffer-load (url buffer) :buffer buffer)))
 
 (defun last-active-buffer ()
   "Return buffer with most recent `last-access'."
   (first (sort-by-time (buffer-list))))
 
 (defun get-inactive-buffers ()
-  "Return inactive buffers sorted by last-access timestamp, or NIL if none."
-  (let ((active-buffers
-          (mapcar #'active-buffer (window-list)))
-        (buffers (buffer-list)))
-    (alex:when-let ((diff (set-difference buffers active-buffers)))
-      ;; Display the most recent inactive buffer.
-      (sort-by-time diff))))
+  "Return inactive buffers sorted by `last-access', when applicable."
+  (alex:when-let ((inactive (set-difference (buffer-list)
+                                            (mapcar #'active-buffer (window-list)))))
+    (sort-by-time inactive)))
 
 (define-command copy-url ()
   "Save current URL to clipboard."
@@ -1672,10 +1663,6 @@ URL-DESIGNATOR is then transformed by BUFFER's `buffer-load-hook'."
     (when new-url
       (check-type new-url quri:uri)
       (setf url new-url)
-      ;; TODO: This condition can be a source of inefficiency.  Besides, it
-      ;; partly duplicates the code in `preprocess-request'.  Can we factor this
-      ;; out?
-
       ;; We could have `on-url-load' and `on-url-unload' methods instead.
       ;; `on-url-unload' could be used to perform some clean up, while
       ;; `on-url-load' would perform the actual loading.
@@ -1714,24 +1701,22 @@ URL-DESIGNATOR is then transformed by BUFFER's `buffer-load-hook'."
 Loads the entry with default `prompter:actions-on-return'."))
 
 (define-class new-url-query ()
-  ((query ""
-          :documentation "Either a URL or a string query passed to `engine'.")
-   (label nil
-          :type (or null string)
-          :documentation "The meaningful text for the query, if query is a URL.")
-   (engine nil
-           :type (or null search-engine)))
+  ((query
+    ""
+    :type (maybe string)
+    :documentation "Either a URL or a string query passed to `engine'.")
+   (label
+    nil
+    :type (maybe string)
+    :documentation "The meaningful text for the query, if query is a URL.")
+   (engine
+    nil
+    :type (maybe search-engine)
+    :documentation "See `search-engine'."))
   (:export-class-name-p t)
   (:export-accessor-names-p t)
-  (:documentation "Structure holding the new URL query generated from a user
- string input.
-If `engine' is set, `query' is passed to it.  See the `url' method.
-If `query' is a valid URL, use it as is.
-If it points to an existing file, 'file://' is automatically prepended to it on
-instantiation.
-If prefixing with 'https://' results in a valid URL, set `query' to this result
-on instantiation.
-Finally, if nothing else, set the `engine' to the `default-search-engine'."))
+  (:documentation "Structure that processes a new URL query from user input.
+Checks whether a valid https or local file URL is requested, in a DWIM fashion."))
 
 (defmethod initialize-instance :after ((query new-url-query)
                                        &key check-dns-p &allow-other-keys)
@@ -1742,9 +1727,12 @@ Finally, if nothing else, set the `engine' to the `default-search-engine'."))
      ;; First check engine: if set, no need to change anything.
      nil)
     ((valid-url-p (query query)
-                  :check-dns-p check-dns-p)
+                  :check-dns-p nil)
      ;; Valid URLs should be passed forward.
      nil)
+    ((and check-dns-p
+          (valid-tld-p (query query)))
+     (setf (query query) (str:concat "https://" (query query))))
     ;; Rest is for invalid URLs:
     ((uiop:file-exists-p (query query))
      (setf (query query)
@@ -1753,11 +1741,6 @@ Finally, if nothing else, set the `engine' to the `default-search-engine'."))
             (uiop:native-namestring
              (uiop:ensure-absolute-pathname
               (query query) *default-pathname-defaults*)))))
-    ((and check-dns-p
-          (valid-url-p (str:concat "https://" (query query))
-                       :check-dns-p check-dns-p))
-     (setf (query query)
-           (str:concat "https://" (query query))))
     (t
      (setf (engine query)
            (or (engine query)
@@ -1798,8 +1781,7 @@ Finally, if nothing else, set the `engine' to the `default-search-engine'."))
                          :query (second completion)
                          :label (first completion)))))
 
-(defun input->queries (input &key (check-dns-p t)
-                               (engine-completion-p))
+(defun input->queries (input &key (check-dns-p t) (engine-completion-p))
   (let* ((terms (sera:tokens input))
          (engines (let ((all-prefixed-engines
                           (remove-if
@@ -1854,32 +1836,20 @@ Finally, if nothing else, set the `engine' to the `default-search-engine'."))
    (prompter:filter-preprocessor
     (lambda (suggestions source input)
       (declare (ignore suggestions source))
-      (input->queries input
-                      :check-dns-p nil
-                      :engine-completion-p nil)))
-   (prompter:filter nil)
+      (input->queries input :check-dns-p t :engine-completion-p nil)))
    (prompter:filter-postprocessor
     (lambda (suggestions source input)
-      (declare (ignore suggestions source))
-      (input->queries input
-                      :check-dns-p t
-                      :engine-completion-p t)))
+      (declare (ignore source))
+      ;; Avoid long computations until the user has finished the query.
+      (sleep 0.15)
+      (append suggestions
+              (input->queries input :check-dns-p nil :engine-completion-p t))))
+   (prompter:filter nil)
    (prompter:actions-on-return #'buffer-load*))
   (:export-class-name-p t)
-  (:documentation "This prompter source tries to \"do the right thing\" to
-generate a new URL query from user input.
-- If the query is a URL, open it directly.
-- If it's a file, prefix the query with 'file://'.
-- If it's a search engine shortcut, include it in the suggestions.
-- If it's none of the above, use the `default-search-engine'.
-
-It runs in two passes.  The first pass does not check the DNS for domain
-validity, nor does it return any search engine suggestions.  This guarantees
-that a good-enough default suggestion is showed instantaneously.
-(We really want this prompter source to be fast!)  The second pass checks the
-DNS to precisely validate domains and returns the search engines suggestions, if
-any.")
-  (:metaclass user-class))
+  (:metaclass user-class)
+  (:documentation "Source listing URL queries from user input in a DWIM fashion.  See
+`new-url-query'."))
 
 (defmethod prompter:object-attributes ((query new-url-query) (source new-url-or-search-source))
   (declare (ignore source))
@@ -1989,8 +1959,7 @@ specified for their contents."
                  :sources (make-instance 'buffer-source))))
   "Reload BUFFER.
 Return it."
-  (when buffer
-    (buffer-load (url buffer) :buffer buffer)))
+  (when buffer (ffi-buffer-reload buffer)))
 
 (define-command reload-current-buffer ()
   "Reload current buffer.
