@@ -49,18 +49,14 @@
    (prompter:constructor *css-colors*)
    (prompter:filter-preprocessor #'prompter:filter-exact-matches)
    (prompter:filter-postprocessor
-    (lambda (suggestions source input)
-      (let ((input-color input))
-        (cons (make-instance 'prompter:suggestion
-                             :value input-color
-                             :attributes (prompter:object-attributes input-color source))
-              suggestions))))
+    (lambda (suggestions source input-color)
+      (unless (str:empty? input-color) (sleep 0.2))
+      (append (when (ignore-errors (cl-colors2:as-rgb input-color))
+                (list (make-instance 'prompter:suggestion
+                                     :value input-color
+                                     :attributes (prompter:object-attributes input-color source))))
+            suggestions)))
    (prompter:actions-on-current-suggestion-enabled-p t)
-   (prompter:actions-on-current-suggestion
-    (lambda (color)
-      (ps-eval :buffer (current-prompt-buffer)
-        (setf (ps:@ (nyxt/ps:qs document "#input") style background-color) (ps:lisp color)
-              (ps:@ (nyxt/ps:qs document "#input") style color) (ps:lisp (theme:contrasting-color color))))))
    (prompter:actions-on-return
     (cons #'identity copy-actions)))
   (:documentation "A source for color search and copying.
@@ -71,7 +67,7 @@ rgb()/hsl() CSS functions representing them."))
   (flet ((display (string)
            (let ((spinneret:*html-style* :tree))
              (spinneret:with-html-string
-               (:span :style (format nil "background-color: ~a; color: ~a; border: 0.1em solid ~a; border-radius: 0.1em"
+               (:span :style (format nil "background-color: ~a; color: ~a; border: 0.1em solid ~a; border-radius: 2px"
                                      color (theme:contrasting-color color) (theme:contrasting-color color))
                       string)))))
     (let ((rgb (cl-colors2:print-css-rgb/a color))
